#!/bin/sh

# some files have to be setuid 0.  they are set here
setuid() {
    for file in \
        /opt/zenoss/bin/zensocket
    do
        chown root:root ${file}
        chmod 4755 ${file}
    done
}

# load up the install functions
export ZENHOME=/opt/zenoss
. ${ZENHOME}/bin/shared-functions.sh
. ${ZENHOME}/bin/install-functions.sh

# kill off any zeo/zope/zen processes that are running
kill_running

# optionally update the bashrc for the user (using the append function)
bashrc=/home/zenoss/.bashrc
append ZENHOME /opt/zenoss ${bashrc}
append PATH '${ZENHOME}/bin:${PATH}' ${bashrc}
append PYTHONPATH /opt/zenoss/lib/python ${bashrc}
append LD_LIBRARY_PATH /opt/zenoss/lib ${bashrc}
chown zenoss ${bashrc}

# run these commands when installing, but not upgrading or reinstalling
if [ $1 -eq 1 ]; then
    # set environment variables
    export OS_USERNAME=zenoss
    export OS_UID=1337
    export OS_HOME=/home/zenoss
    export OS_SHELL=/bin/bash

    # set up symlinks for all the runlevels
    chkconfig --add zenoss

    # update the files to be setuid
    setuid

    # set the flag to identify this as a fresh install (rather than an
    # upgrade)
    touch ${ZENHOME}/.fresh_install
fi

# run these commands when upgrading (but not installing or reinstalling)
if [ $1 -eq 2 ]; then
    # set the flag to identify this as an upgrade (rather than a fresh
    # install)
    touch ${ZENHOME}/.upgraded

    # update the files to be setuid
    setuid

    # remove the .pyc files that were written by python while zenoss ran
    find ${ZENHOME} -name \*.pyc -exec rm -f '{}' \;
fi

# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=